/*----------------------------------------------------------------------------
 * Name:    Blinky.c
 * Purpose: LED Flasher
 *----------------------------------------------------------------------------
 * This file is part of the uVision/ARM development tools.
 * This software may only be used under the terms of a valid, current,
 * end user licence from KEIL for a compatible version of KEIL software
 * development tools. Nothing else gives you the right to use this software.
 *
 * This software is supplied "AS IS" without warranties of any kind.
 *
 * Copyright (c) 2018 Keil - An ARM Company. All rights reserved.
 *----------------------------------------------------------------------------*/

#include <stdio.h>

#include "cmsis_os2.h"                  // ::CMSIS:RTOS2

#include "RTE_Components.h"             // Component selection

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

#include "fsl_gpio.h"
#include "pin_mux.h"


// Main stack size must be multiple of 8 Bytes
#define APP_MAIN_STK_SZ                (1024U)
uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};

static osThreadId_t tid_led;            // Thread id of thread: LED
static osThreadId_t tid_btn;            // Thread id of thread: Buttons


/*----------------------------------------------------------------------------
  ThreadLED: blink LED
 *----------------------------------------------------------------------------*/
__NO_RETURN static void ThreadLED(void *argument) {

  (void)argument;

  for (;;) {
    osThreadFlagsWait(0x0001U, osFlagsWaitAny ,osWaitForever);
    /* Turn specified LED on */
    GPIO_PinWrite(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, 0U);
    osThreadFlagsWait(0x0001U, osFlagsWaitAny ,osWaitForever);
    /* Turn specified LED off */
    GPIO_PinWrite(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, 1U);
  }

}

/*----------------------------------------------------------------------------
  ThreadButtons: handle button state
 *----------------------------------------------------------------------------*/
__NO_RETURN static void ThreadButtons(void *argument) {

  (void)argument;

  for (;;) {
    osDelay(500U);
    /* Wait while holding USER button */
    while (!GPIO_PinRead(BOARD_INITBUTTONS_USER_BUTTON_PORT, BOARD_INITBUTTONS_USER_BUTTON_GPIO_PIN));
    osThreadFlagsSet(tid_led, 0x0001U);
  }

}

/*----------------------------------------------------------------------------
 * Application main thread
 *---------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *argument) {
  gpio_pin_config_t led_pin_config = {kGPIO_DigitalOutput, 0, kGPIO_NoIntmode};
  gpio_pin_config_t btn_pin_config = {kGPIO_DigitalInput,  0, kGPIO_NoIntmode};

  (void)argument;

#ifdef RTE_Compiler_EventRecorder
  EventRecorderEnable (EventRecordError, 0xF0U, 0xF8U); // RTOS Events
  EventRecorderEnable (EventRecordAll,   0xF2U, 0xF2U); // Thread Events
#endif

  GPIO_PinInit(BOARD_INITLED_USER_LED_PORT,        BOARD_INITLED_USER_LED_GPIO_PIN,        &led_pin_config);
  GPIO_PinInit(BOARD_INITBUTTONS_USER_BUTTON_PORT, BOARD_INITBUTTONS_USER_BUTTON_GPIO_PIN, &btn_pin_config);

  tid_btn = osThreadNew (ThreadButtons, NULL, NULL);    // Create buttons thread
  if (tid_btn == NULL) {
    // add error handling if thread creation failed
  }

  tid_led = osThreadNew (ThreadLED, NULL, NULL);        // Create LED thread
  if (tid_led == NULL) {
    // add error handling if thread creation failed
  }

  for (;;) {
    osThreadFlagsWait (0, osFlagsWaitAny, osWaitForever);
  }
}
