/*-----------------------------------------------------------------------------
 * Name:    LED_MIMXRT1050-EVK.c
 * Purpose: LED interface for NXP MIMXRT1050-EVK Board
 * Rev.:    1.0.0
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2018 Arm Limited

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/


#include "Board_LED.h"

#include "fsl_gpio.h"
#include "pin_mux.h"


#define LED_COUNT                      (1)


const gpio_pin_config_t user_led_pin_cfg   = { kGPIO_DigitalOutput, 1U, kGPIO_NoIntmode };
const gpio_pin_config_t user_led_pin_uncfg = { kGPIO_DigitalInput,  0U, kGPIO_NoIntmode };


/**
  \fn          int32_t LED_Initialize (void)
  \brief       Initialize I/O interface for LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Initialize (void) {

  GPIO_PinInit(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, &user_led_pin_cfg);

  return 0;
}

/**
  \fn          int32_t LED_Uninitialize (void)
  \brief       De-initialize I/O interface for LEDs
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Uninitialize (void) {

  GPIO_PinInit(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, &user_led_pin_uncfg);

  return 0;
}

/**
  \fn          int32_t LED_On (uint32_t num)
  \brief       Turn on a single LED indicated by num
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_On (uint32_t num) {

  if (num >= LED_COUNT) {
    return -1;
  }

  GPIO_PinWrite(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, 0U);

  return 0;
}

/**
  \fn          int32_t LED_Off (uint32_t num)
  \brief       Turn off a single LED indicated by num
  \param[in]   num  LED number
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_Off (uint32_t num) {

  if (num >= LED_COUNT) {
    return -1;
  }

  GPIO_PinWrite(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, 1U);

  return 0;
}

/**
  \fn          int32_t LED_SetOut (uint32_t val)
  \brief       Control all LEDs with the bit vector val
  \param[in]   val  each bit represents the status of one LED.
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t LED_SetOut (uint32_t val) {

  if ((val & 1U) != 0U) {
    GPIO_PinWrite(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, 0U);
  } else {
    GPIO_PinWrite(BOARD_INITLED_USER_LED_PORT, BOARD_INITLED_USER_LED_GPIO_PIN, 1U);
  }

  return 0;
}

/**
  \fn          uint32_t LED_GetCount (void)
  \brief       Get number of available LEDs on evaluation hardware
  \return      Number of available LEDs
*/
uint32_t LED_GetCount (void) {

  return LED_COUNT;
}
