/*------------------------------------------------------------------------------
 * MDK Middleware
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    retarget_user.c
 * Purpose: User specific retarget of stdin, stdout and stderr
 *----------------------------------------------------------------------------*/

#include <stdint.h>
#include "fsl_log.h"                    // NXP::Device:SDK Utilities:debug_console

/**
  Put a character to the stderr
 
  \param[in]   ch  Character to output
  \return          The character written, or -1 on write error.
*/
int stderr_putchar (int ch) {
    /* Send data. */
    return LOG_Push((uint8_t *)(&ch), 1);
}

/**
  Put a character to the stdout
 
  \param[in]   ch  Character to output
  \return          The character written, or -1 on write error.
*/
int stdout_putchar (int ch) {
    /* Send data. */
    return LOG_Push((uint8_t *)(&ch), 1);
}

/**
  Get a character from the stdio
 
  \return     The next character from the input, or -1 on read error.
*/
int stdin_getchar (void) {
  int32_t ch;
 
  do {
    /* Receive data */
    LOG_ReadCharacter((uint8_t*)&ch);
  } while (ch == -1);
  return (ch);
}
