/*------------------------------------------------------------------------------
 * MDK Middleware - Component ::USB:Device
 * Copyright (c) 2004-2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    HID.c
 * Purpose: USB Device - Human Interface Device example
 *----------------------------------------------------------------------------*/

#include "RTE_Components.h"             // Component selection

#include "rl_usb.h"                     /* RL-USB function prototypes         */

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

#include "board.h"                      // Keil::Board Support:SDK Project Template:Project_Template
#include "clock_config.h"               // Keil::Board Support:SDK Project Template:Project_Template
#include "peripherals.h"                // Keil::Board Support:SDK Project Template:Project_Template
#include "pin_mux.h"                    // Keil::Board Support:SDK Project Template:Project_Template
#include "fsl_gpio.h"

// Main stack size must be multiple of 8 Bytes
#define APP_MAIN_STK_SZ (2048U)
uint64_t app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};

/*------------------------------------------------------------------------------
 *        Application
 *----------------------------------------------------------------------------*/

__NO_RETURN void app_main (void *arg) {
  gpio_pin_config_t led_pin_config = {kGPIO_DigitalOutput, 0, kGPIO_NoIntmode};
  gpio_pin_config_t btn_pin_config = {kGPIO_DigitalInput,  0, kGPIO_NoIntmode};
  uint8_t but, but_last;

  (void)arg;

#ifdef RTE_Compiler_EventRecorder
  EventRecorderEnable (EventRecordError, 0xA0U, 0xA6U);  /* USBD Error Events */
  EventRecorderEnable (EventRecordAll  , 0xA0U, 0xA0U);  /* Core Events */
  EventRecorderEnable (EventRecordAll  , 0xA5U, 0xA5U);  /* HID Events */
#endif

  GPIO_PinInit(BOARD_INITLED_USER_LED_PORT,        BOARD_INITLED_USER_LED_GPIO_PIN,        &led_pin_config);
  GPIO_PinInit(BOARD_INITBUTTONS_USER_BUTTON_PORT, BOARD_INITBUTTONS_USER_BUTTON_GPIO_PIN, &btn_pin_config);

  USBD_Initialize         (0);          /* USB Device 0 Initialization        */
  USBD_Connect            (0);          /* USB Device 0 Connect               */

  for(;;) {                             /* Loop forever                       */
    but = GPIO_PinRead(BOARD_INITBUTTONS_USER_BUTTON_PORT, BOARD_INITBUTTONS_USER_BUTTON_GPIO_PIN) ^ 1U;
    if (but != but_last) {
      but_last = but;
      USBD_HID_GetReportTrigger(0, 0, &but, 1);
    }
    osDelay(100);                       /* 100 ms delay for sampling buttons  */
  }
}
