/*------------------------------------------------------------------------------
 * Application template
 * Copyright (c) 2018 ARM Germany GmbH. All rights reserved.
 *------------------------------------------------------------------------------
 * Name:    main.c
 * Purpose: Main module
 *----------------------------------------------------------------------------*/

#include "RTE_Components.h"

#include "cmsis_os2.h"                  // ::CMSIS:RTOS2

#include "peripherals.h"                // Keil::Board Support:SDK Project Template:Project_Template
#include "pin_mux.h"                    // Keil::Board Support:SDK Project Template:Project_Template
#include "clock_config.h"               // Keil::Board Support:SDK Project Template:Project_Template
#include "board.h"                      // Keil::Board Support:SDK Project Template:Project_Template

#ifdef RTE_Compiler_EventRecorder
#include "EventRecorder.h"
#endif

// Main stack size must be multiple of 8 Bytes
#define APP_MAIN_STK_SZ                (1024U)
      uint64_t       app_main_stk[APP_MAIN_STK_SZ / 8];
const osThreadAttr_t app_main_attr = {
  .stack_mem  = &app_main_stk[0],
  .stack_size = sizeof(app_main_stk)
};


/*------------------------------------------------------------------------------
 * Application main thread
 *----------------------------------------------------------------------------*/
__NO_RETURN void app_main (void *argument) {

  (void)argument;

#ifdef RTE_Compiler_EventRecorder
  EventRecorderEnable(EventRecordError, 0xF0U, 0xF8U);  // RTOS Events
  EventRecorderEnable(EventRecordAll,   0xF2U, 0xF2U);  // Thread Events
#endif

  for (;;) {
     // Add application code here
  }
}


/*------------------------------------------------------------------------------
 * main function
 *----------------------------------------------------------------------------*/
int main(void) {

  // System initialization
  BOARD_InitBootPeripherals();
  BOARD_InitBootPins();
  BOARD_InitBootClocks();
  BOARD_InitDebugConsole();

  // Update System Core Clock info
  SystemCoreClockUpdate();

#ifdef RTE_Compiler_EventRecorder
  // Initialize and start Event Recorder
  EventRecorderInitialize(EventRecordError, 1U);
#endif
  // ...

  osKernelInitialize();                         // Initialize CMSIS-RTOS2
  osThreadNew(app_main, NULL, &app_main_attr);  // Create application main thread
  osKernelStart();                              // Start thread execution

  for (;;) {}
}
